@@ -4,7 +4,7 @@
 
 #' Sensitivity analysis for unmeasured confounding in meta-analyses
 #' 
-#' This function implements the sensitivity analyses of Mathur & VanderWeele (2019) and Mathur & VanderWeele (2020). It computes point estimates, standard errors, and confidence intervals
+#' This function implements the sensitivity analyses of Mathur & VanderWeele (2020a, 2020b). It computes point estimates, standard errors, and confidence intervals
 #' for (1) \code{Prop}, the proportion of studies with true causal effect sizes above or below a chosen threshold \code{q} as a function of the bias parameters;
 #' (2) the minimum bias factor on the relative risk scale (\code{Tmin}) required to reduce to
 #' less than \code{r} the proportion of studies with true causal effect sizes more extreme than
@@ -44,16 +44,18 @@
 #' 
 #' @param vt2 Estimated variance of \eqn{\tau^2} from confounded meta-analysis. Only used if \code{method = "parametric"}.
 #' 
+#' @param \dots Additional arguments passed to \code{confounded_meta}.
+#' 
 #' @export
 #' @details
 #' ## Specifying the sensitivity parameters on the bias
-#' By convention, the average log-bias factor, \code{muB}, is taken to be greater than 0 (Mathur & VanderWeele, 2019; Ding & VanderWeele, 2017). Confounding can operate on average either away from or toward the null, a choice specified via \code{muB.toward.null}. The most common choice for sensitivity analysis is to consider bias that operates on average away from the null, which is \code{confounded_meta}'s default. In such an analysis, correcting for the bias involves shifting studies' estimates back toward the null by \code{muB} (i.e., if \code{yr > 0}, the estimates will be corrected downward; if \code{yr < 0}, they will be corrected upward). Alternatively, to consider bias that operates on average away from the null, you would still specify \code{muB > 0} but would also specify \code{muB.toward.null = TRUE}. For detailed guidance on choosing the sensitivity parameters \code{muB} and \code{sigB}, see Section 5 of Mathur & VanderWeele (2019).
+#' By convention, the average log-bias factor, \code{muB}, is taken to be greater than 0 (Mathur & VanderWeele, 2020a; Ding & VanderWeele, 2017). Confounding can operate on average either away from or toward the null, a choice specified via \code{muB.toward.null}. The most common choice for sensitivity analysis is to consider bias that operates on average away from the null, which is \code{confounded_meta}'s default. In such an analysis, correcting for the bias involves shifting studies' estimates back toward the null by \code{muB} (i.e., if \code{yr > 0}, the estimates will be corrected downward; if \code{yr < 0}, they will be corrected upward). Alternatively, to consider bias that operates on average away from the null, you would still specify \code{muB > 0} but would also specify \code{muB.toward.null = TRUE}. For detailed guidance on choosing the sensitivity parameters \code{muB} and \code{sigB}, see Section 5 of Mathur & VanderWeele (2020a).
 #' 
 #' ## Specifying the threshold \code{q}
-#' For detailed guidance on choosing the threshold \code{q}, see the Supplement of Mathur & VanderWeele (2019).   
+#' For detailed guidance on choosing the threshold \code{q}, see the Supplement of Mathur & VanderWeele (2020a).   
 #' 
 #' ## Specifying the estimation method
-#' By default, \code{confounded_meta} performs estimation using a \strong{calibrated method} (Mathur & VanderWeele, 2020) that extends work by Wang et al. (2019). This method makes no assumptions about the distribution of population effects and performs well in meta-analyses with as few as 10 studies, and performs well even when the proportion being estimated is close to 0 or 1. However, it only accommodates bias whose strength is the same in all studies (homogeneous bias). When using this method, the following arguments need to be specified:
+#' By default, \code{confounded_meta} performs estimation using a \strong{calibrated method} (Mathur & VanderWeele, 2020b) that extends work by Wang et al. (2019). This method makes no assumptions about the distribution of population effects and performs well in meta-analyses with as few as 10 studies, and performs well even when the proportion being estimated is close to 0 or 1. However, it only accommodates bias whose strength is the same in all studies (homogeneous bias). When using this method, the following arguments need to be specified:
 #' \itemize{
 #'  \item \code{q}
 #'  \item \code{r} (if you want to estimate \code{Tmin} and \code{Gmin})
@@ -79,8 +81,13 @@
 #' ## Effect size measures other than log-relative risks
 #' If your meta-analysis uses effect sizes other than log-relative risks, you should first approximately convert them to log-relative risks, for example via [EValue::convert_measures()] and then pass the converted point estimates or meta-analysis estimates to \code{confounded_meta}. 
 #' 
+#' ## Interpreting \code{Tmin} and \code{Gmin}
+#' \code{Tmin} is defined as the minimum bias factor on the relative risk scale, common to all meta-analyzed studies, that would be required to reduce to less than \code{r} the proportion of studies with true causal effect sizes stronger than the threshold \code{q}. \code{Gmin} is defined as the minimum confounding strength on the relative risk scale -- that is, the relative risk relating unmeasured confounder(s) to both the exposure and the outcome -- common to all meta-analyzed studies, that would be required to reduce to less than \code{r} the proportion of studies with true causal effect sizes stronger than the threshold \code{q}. \code{Gmin} is a one-to-one transformation of \code{Gmin} given by \eqn{Gmin = Tmin + \sqrt{Tmin * (Tmin - 1)} }. If the estimated proportion of meaningfully strong effect sizes is already less than \code{r} even without the introduction of any bias, \code{Tmin} and \code{Gmin} will be set to 1.
+#' 
+#' The direction of bias represented by \code{Tmin} and \code{Gmin} is dependent on the argument \code{tail}: when \code{tail = "above"}, these metrics consider bias that had operated to \emph{increase} studies' point estimates, and when \code{tail = "below"}, these metrics consider bias that had operated to \emph{decrease} studies' point estimates. Such bias could operate toward or away from the null depending on whether the pooled point estimate \code{yr} happens to fall above or below the null. As such, the direction of bias represented by \code{Tmin} and \code{Gmin} may or may not match that specified by the argument \code{muB.toward.null} (which is used only for estimation of \code{Prop}).
+#' 
 #' ## When these methods should be used
-#' These methods perform well only in meta-analyses with at least 10 studies; we do not recommend reporting them in smaller meta-analyses. Additionally, it only makes sense to consider proportions of effects stronger than a threshold when the heterogeneity estimate \code{t2} is greater than 0. For meta-analyses with fewer than 10 studies or with a heterogeneity estimate of 0, you can simply report E-values for the point estimate via [EValue::evalue()] (VanderWeele & Ding, 2017; see Mathur & VanderWeele (2019), Section 7.2 for interpretation in the meta-analysis context).
+#' These methods perform well only in meta-analyses with at least 10 studies; we do not recommend reporting them in smaller meta-analyses. Additionally, it only makes sense to consider proportions of effects stronger than a threshold when the heterogeneity estimate \code{t2} is greater than 0. For meta-analyses with fewer than 10 studies or with a heterogeneity estimate of 0, you can simply report E-values for the point estimate via [EValue::evalue()] (VanderWeele & Ding, 2017; see Mathur & VanderWeele (2020a), Section 7.2 for interpretation in the meta-analysis context).
 #'  
 #' 
 #' @keywords meta-analysis
@@ -91,9 +98,9 @@
 #' boot
 #' 
 #' @references
-#' Mathur MB & VanderWeele TJ (2019). Sensitivity analysis for unmeasured confounding in meta-analyses. \emph{Journal of the American Statistical Association}.
+#' Mathur MB & VanderWeele TJ (2020a). Sensitivity analysis for unmeasured confounding in meta-analyses. \emph{Journal of the American Statistical Association}.
 #' 
-#' Mathur MB & VanderWeele TJ (2020). Robust metrics and sensitivity analyses for meta-analyses of heterogeneous effects. \emph{Epidemiology}.
+#' Mathur MB & VanderWeele TJ (2020b). Robust metrics and sensitivity analyses for meta-analyses of heterogeneous effects. \emph{Epidemiology}.
 #' 
 #' Mathur MB & VanderWeele TJ (2019). New statistical metrics for meta-analyses of heterogeneous effects. \emph{Statistics in Medicine}.
 #'
@@ -212,7 +219,10 @@ confounded_meta = function( method="calibrated",  # for both methods
                             yr = NA,
                             vyr = NA,
                             t2 = NA,
-                            vt2 = NA
+                            vt2 = NA,
+                            
+                            ...
+                          
 ) {
   
   
@@ -231,13 +241,33 @@ confounded_meta = function( method="calibrated",  # for both methods
   # yi.name = "yi"
   # vi.name = "vyi"
   
+  ##### Look for Additional Passed Arguments #####
+  dots = list(...)
   
-  ##### Check for Bad Input - Common to Parametric and Calibrated Methods #####
+  # should warnings be simplified for use with the Shiny website?
+  if ( "simplifyWarnings" %in% names(dots) ){
+    simplifyWarnings =  dots$simplifyWarnings
+  } else {
+    simplifyWarnings = FALSE
+  }
+  
+  
+  ##### Check for Bad or Incomplete Input - Common to Parametric and Calibrated Methods #####
   if ( ! is.na(r) ) {
     if (r < 0 | r > 1) stop("r must be between 0 and 1")
   }
   
-  if ( is.na(r) ) message("Cannot compute Tmin or Gmin without r. Returning only prop.")
+  if ( is.na(r) ){
+    if ( simplifyWarnings == FALSE ) message("Cannot compute Tmin or Gmin without r. Returning only prop.")
+    if ( simplifyWarnings == TRUE ) message("Cannot compute bias or confounding strength required to shift to r unless you provide r. Returning only the proportion.")
+  }
+  
+  if ( !is.na(muB) & muB < 0 ) {
+    if ( simplifyWarnings == FALSE ) stop("Must have muB > 0. Use the muB.toward.null argument instead if you want to consider bias away from the null. See Details.")
+    
+    if ( simplifyWarnings == TRUE ) stop("Must have muB > 0.")
+    
+  }
   
   
   ##### PARAMETRIC #####
@@ -251,7 +281,7 @@ confounded_meta = function( method="calibrated",  # for both methods
     
     # the second condition is needed for Shiny app:
     #  if user deletes the input in box, then it's NA instead of NULL
-    if ( ! is.na(vyr) ) {
+    if ( !is.na(vyr) ) {
       if (vyr < 0) stop("Variance of point estimate cannot be negative")
     }
     
@@ -260,87 +290,129 @@ confounded_meta = function( method="calibrated",  # for both methods
     }
     
     if ( !is.na(sigB) ) {
-      if ( t2 <= sigB^2 ) stop("Must have t2 > sigB^2")
+      if ( t2 <= sigB^2 ) {
+        if (simplifyWarnings == FALSE) stop("Must have t2 > sigB^2")
+        
+        # on the website, this can only happen if they set t2=0 but proportion due to confounding > 0
+        if (simplifyWarnings == TRUE) stop("Cannot have a nonzero proportion of heterogeneity due to variation in confounding bias when there is no heterogeneity to begin with")
+      }
       if ( sigB < 0 ) stop("Bias factor standard deviation cannot be negative")
     }
     
-    if ( !is.na(muB) & muB < 0 ) {
-      stop("Must have muB > 0. Use the muB.toward.null argument instead if you want to consider bias away from the null. See Details.")
-    }
     
     
     ##### Messages When Not All Output Can Be Computed #####
-    if ( is.na(vyr) | is.na(vt2) ) message("Cannot compute inference without vyr and vt2. Returning only point estimates.")
+    if ( is.na(vyr) | is.na(vt2) ) message("Cannot compute inference without variances of point estimate and of heterogeneity estimate. Returning only point estimates.")
     
     ##### Point Estimates: Causative Case #####
     # if tail isn't provided, assume user wants the more extreme one (away from the null)
     if ( is.na(tail) ) {
-      tail = ifelse( yr > log(1), "above", "below" )
+      tail = ifelse( yr > 0, "above", "below" )
       warning( paste( "Assuming you want tail =", tail, "because it wasn't specified") )
     }
     
     # bias-corrected mean
     # usual case: bias that went away from null, so correction shifts toward null
-    if ( muB.toward.null == FALSE & yr > log(1) ) yr.corr = yr - muB
-    if ( muB.toward.null == FALSE & yr < log(1) ) yr.corr = yr + muB
+    if ( muB.toward.null == FALSE & yr > 0 ) yr.corr = yr - muB
+    if ( muB.toward.null == FALSE & yr < 0 ) yr.corr = yr + muB
     # less-usual case: bias that went toward null, so correction shifts away from null
-    if ( muB.toward.null == TRUE & yr > log(1) ) yr.corr = yr + muB
-    if ( muB.toward.null == TRUE & yr < log(1) ) yr.corr = yr - muB
+    if ( muB.toward.null == TRUE & yr > 0 ) yr.corr = yr + muB
+    if ( muB.toward.null == TRUE & yr < 0 ) yr.corr = yr - muB
     
     
     if ( tail == "above" ) {
       
+      # point estimate for Phat
       if ( !is.na(muB) & !is.na(sigB) ) {
-        # prop above
+        # point estimate for Phat
         Z = ( q - yr.corr ) / sqrt( t2 - sigB^2 )
         Phat = 1 - pnorm(Z) 
       } else {
         Phat = NA
       }
       
+      # point estimates for Tmin, Gmin
       if ( !is.na(r) ) {
         
-        # min bias factor
-        # the max is there in case no bias is needed
-        # (i.e., proportion of effects > q already < r without confounding)
-        Tmin = max( 1, exp( qnorm(1-r) * sqrt(t2) - q + yr ) )
+        # first check if any shifting is actually needed
+        # current Phat with no bias
+        Phat.naive = 1 - pnorm( (q - yr) / sqrt(t2) )
+        
+        if ( Phat.naive <= r ) {
+          Tmin = 1
+        } else {
+          # min bias factor
+          # the max is there in case no bias is needed
+          # (i.e., the bias would be going in the other direction)
+          # (i.e., proportion of effects > q already < r without confounding)
+          Tmin = max( 1, exp( qnorm(1-r) * sqrt(t2) - q + yr ) )
+          
+          # alternative way of handling this issue:
+          # Tmin could be less than 1 if yr has to be shifted POSITIVELY
+          #  rather than negatively to achieve r
+          #  e.g., yr^c = log(0.5), q = log(1s.5), r = 0.75
+          # for consistency with calibrated output, take Tmin's inverse so it's always positive
+          #if ( Tmin < 1 ) Tmin = 1 / Tmin
+        }
         
         # min confounding strength
         # suppress warnings to avoid warnings about NaN when term inside sqrt is negative
         Gmin = suppressWarnings( Tmin + sqrt( Tmin^2 - Tmin ) )
-      } else {
+      }
+      
+      if ( is.na(r) ) {
         Tmin = Gmin = NA
       }
-    }
+    } # end tail = "above"
     
     ##### Point Estimates: Preventive Case #####
     if ( tail == "below" ) {
       
+      # point estimate for Phat
       if ( !is.na(muB) & !is.na(sigB) ) {
-        # prop below
         Z = ( q - yr.corr ) / sqrt( t2 - sigB^2 )
         Phat = pnorm(Z) 
       } else {
         Phat = NA
       }
-      
+     
+      # point estimates for Tmin, Gmin
       if ( !is.na(r) ) {
-        # min bias factor
-        Tmin = exp( q - yr - qnorm(r) * sqrt(t2) )
+        
+        # first check if any shifting is actually needed
+        # current Phat with no bias
+        Phat.naive = pnorm( (q - yr) / sqrt(t2) )
+        
+        if ( Phat.naive <= r ) {
+          Tmin = 1
+        } else {
+          # the max is there in case no bias is needed
+          Tmin = max( 1, exp( q - yr - qnorm(r) * sqrt(t2) ) )
+          
+          # alternative way of handling this issue:
+          # # Tmin could be less than 1 if yr has to be shifted NEGATIVELY
+          # #  rather than positively to achieve r
+          # #  e.g., yr^c = log(1.5), q = log(0.5), r = 0.75
+          # # for consistency with calibrated output, take Tmin's inverse so it's always positive
+          # if ( Tmin < 1 ) Tmin = 1 / Tmin
+        }
         
         # min confounding strength
         Gmin = suppressWarnings( Tmin + sqrt( Tmin^2 - Tmin ) )
-      } else {
+      }
+      
+      if ( is.na(r) ) {
         Tmin = Gmin = NA
       }
-    }
+      
+    } # end tail = "below"
     
     ##### Delta Method Inference: P-Hat #####
     # do inference only if given needed SEs
     if ( !is.na(vyr) & !is.na(vt2) & !is.na(muB) & !is.na(sigB) ){
       
-      # term in numerator depends on whether causative or preventive RR
-      num.term = ifelse( yr > log(1), q + muB - yr, q - muB - yr )
+      # term in numerator depends tail
+      num.term = ifelse( tail == "above", q + muB - yr, q - muB - yr )
       
       term1.1 = vyr / (t2 - sigB^2 )
       term1.2 = ( vt2 * (num.term)^2 ) / ( 4 * (t2 - sigB^2 )^3 )
@@ -356,7 +428,11 @@ confounded_meta = function( method="calibrated",  # for both methods
       
       
       # warn if bootstrapping needed
-      if ( Phat < 0.15 | Phat > 0.85 ) warning('Prop is close to 0 or 1. We recommend choosing method = \"calibrated\" or alternatively using bias-corrected and accelerated bootstrapping to estimate all inference in this case.')
+      if ( Phat < 0.15 | Phat > 0.85 ) {
+        if ( simplifyWarnings == FALSE) warning('Prop is close to 0 or 1. We recommend choosing method = \"calibrated\" or alternatively using bias-corrected and accelerated bootstrapping to estimate all inference in this case.')
+        
+       # no warning for the simplifyWarnings == TRUE case because website has its own version 
+      }
       
     } else {
       SE.Phat = lo.Phat = hi.Phat = NA
@@ -364,12 +440,15 @@ confounded_meta = function( method="calibrated",  # for both methods
     
     ##### Delta Method Inference: Tmin and Gmin #####
     # do inference only if given needed SEs and r
-    if ( !is.na(vyr) & !is.na(vt2) & !is.na(r) ){
+    # last condition: if Tmin has been set to 1, give NAs for inference
+    if ( !is.na(vyr) & !is.na(vt2) & !is.na(r) & Tmin != 1 ){
       
       ##### Tmin #####
-      if (yr > log(1) ) {
+      if ( tail == "above" ) {
+        
         term = ( vt2 * qnorm(1-r)^2 ) / ( 4 * t2 )
         SE.T = exp( sqrt(t2) * qnorm(1-r) - q + yr ) * sqrt( vyr + term  )
+        
       } else {
         term = ( vt2 * qnorm(r)^2 ) / ( 4 * t2 )
         SE.T = exp( q - yr - sqrt(t2) * qnorm(r) ) * sqrt( vyr + term  )
@@ -386,10 +465,11 @@ confounded_meta = function( method="calibrated",  # for both methods
       lo.G = max( 1, Gmin + qnorm( tail.prob )*SE.G )  # confounding RR can't be < 1
       hi.G = Gmin - qnorm( tail.prob )*SE.G  # but has no upper bound
       
-    } else {  # i.e., user didn't pass parameters needed for inference
+    } else {  # i.e., user didn't pass parameters needed for inference, or else Tmin = 1
       SE.T = SE.G = lo.T = lo.G = hi.T = hi.G = NA
     }
     
+    
   } # closes parametric method
   
   ##### CALIBRATED #####
@@ -402,7 +482,7 @@ confounded_meta = function( method="calibrated",  # for both methods
       calib = calib_ests( yi = dat[[yi.name]], 
                           sei = sqrt( dat[[vi.name]] ) )
       
-      tail = ifelse( median(calib) > log(1), "above", "below" )
+      tail = ifelse( median(calib) > 0, "above", "below" )
       warning( paste( "Assuming you want tail =", tail, "because it wasn't specified") )
     }
     
@@ -448,6 +528,7 @@ confounded_meta = function( method="calibrated",  # for both methods
                                   q = q,
                                   tail = tail,
                                   dat = dat,
+                                  muB.toward.null = muB.toward.null,
                                   yi.name = yi.name,
                                   vi.name = vi.name,
                                   CI.level = CI.level)
@@ -457,7 +538,7 @@ confounded_meta = function( method="calibrated",  # for both methods
       SE.Phat = as.numeric( Phat.CI.lims[3] )
       
       if ( any( is.na( c(lo.Phat, hi.Phat, SE.Phat) ) ) ) {
-        message("The confidence interval and/or standard error for the proportion were not estimable via bias-corrected and accelerated bootstrapping. You can try increasing R.")
+        message("The confidence interval and/or standard error for the proportion were not estimable via bias-corrected and accelerated bootstrapping. You can try increasing the number of bootstrap iterates or choosing a less extreme threshold.")
       }
       
       Tmin.Gmin.CI.lims = Tmin_Gmin_CI_lims( R,
@@ -476,9 +557,9 @@ confounded_meta = function( method="calibrated",  # for both methods
       hi.G = as.numeric( Tmin.Gmin.CI.lims["hi.G"] )
       SE.G = as.numeric( Tmin.Gmin.CI.lims["SE.G"] )
       
-      
-      if ( any( is.na( c(lo.T, hi.T, SE.T, lo.G, hi.G, SE.G) ) ) ) {
-        message("The confidence interval and/or standard error for Tmin and Gmin were not estimable via bias-corrected and accelerated bootstrapping. You can try increasing R.")
+      # last condition is because we don't actually do bootstrapping if Tmin = 1
+      if ( any( is.na( c(lo.T, hi.T, SE.T, lo.G, hi.G, SE.G) ) ) & ( !is.na(Tmin) & Tmin != 1 ) ) {
+        message("The confidence interval and/or standard error for Tmin and Gmin were not estimable via bias-corrected and accelerated bootstrapping. You can try increasing the number of bootstrap iterates or choosing a less extreme threshold.")
       }
       
     }  # closes "if ( !is.na(r) )"
@@ -488,7 +569,10 @@ confounded_meta = function( method="calibrated",  # for both methods
   ##### Messages about Results #####
   if ( exists("Tmin") ) {
     if ( !is.na(Tmin) & Tmin == 1 ) {
-      message("Prop is already less than or equal to r even with no confounding, so Tmin and Gmin are simply equal to 1. No confounding at all is required to make the specified shift.")
+      if (simplifyWarnings == FALSE) message("Prop is already less than or equal to r even with no confounding, so Tmin and Gmin are simply equal to 1. No confounding at all is required to make the specified shift.")
+      
+      # no warning for website because it produces its own
+      #if (simplifyWarnings == TRUE) message("The proportion is already less than or equal to r even with no confounding, so the amount of bias and of confounding strength required to make the specified shift are simply equal to 1. No confounding at all is required to make the specified shift.")
     }
     
     if ( !is.na(Tmin) & muB.toward.null == TRUE ) {
@@ -496,9 +580,7 @@ confounded_meta = function( method="calibrated",  # for both methods
     }
   }  
   
-  
-  
-  
+
   ##### Return Results #####
   return( data.frame( Value = c("Prop", "Tmin", "Gmin"), 
                       Est = c( Phat, Tmin, Gmin ),
@@ -561,9 +643,9 @@ confounded_meta = function( method="calibrated",  # for both methods
 #' ggplot2
 #' @importFrom dplyr %>% rowwise mutate rename
 #' @references
-#' Mathur MB & VanderWeele TJ (2020). Robust metrics and sensitivity analyses for meta-analyses of heterogeneous effects. \emph{Epidemiology}.
-#'
-#' Mathur MB & VanderWeele TJ (2020). Sensitivity analysis for unmeasured confounding in meta-analyses. \emph{Journal of the American Statistical Association}.
+#' Mathur MB & VanderWeele TJ (2020a). Sensitivity analysis for unmeasured confounding in meta-analyses. \emph{Journal of the American Statistical Association}.
+#' 
+#' Mathur MB & VanderWeele TJ (2020b). Robust metrics and sensitivity analyses for meta-analyses of heterogeneous effects. \emph{Epidemiology}.
 #'
 #' Wang C-C & Lee W-C (2019). A simple method to estimate prediction intervals and
 #' predictive distributions: Summarizing meta-analyses
@@ -592,7 +674,7 @@ confounded_meta = function( method="calibrated",  # for both methods
 #' # # commented out because takes a while too run
 #' # sens_plot( method = "calibrated",
 #' #            type="line",
-#' #            q=log(1),
+#' #            q=0,
 #' #            tail = "below",
 #' #            dat = toyMeta,
 #' #            yi.name = "est",
@@ -643,7 +725,7 @@ confounded_meta = function( method="calibrated",  # for both methods
 #'            vt2 = .001,
 #'            t2=0.4,
 #'            sigB = 0.1,
-#'            Bmin=log(1),
+#'            Bmin=0,
 #'            Bmax=log(4) )
 #' 
 #' ##### Distribution Line Plot #####
@@ -671,7 +753,7 @@ sens_plot = function(method="calibrated",
                      tail=NA,
                      muB.toward.null = FALSE,
                      give.CI=TRUE,
-                     Bmin = log(1),
+                     Bmin = 0,
                      Bmax = log(4),
                      breaks.x1=NA,
                      breaks.x2=NA,
@@ -708,7 +790,7 @@ sens_plot = function(method="calibrated",
   # dat = d
   # yi.name = "yi"
   # vi.name = "vi"
-  # Bmin = log(1)
+  # Bmin = 0
   # Bmax = log(5)
   # CI.level = 0.95
   # tail = "above"
@@ -725,7 +807,7 @@ sens_plot = function(method="calibrated",
   # t2 = 0.3
   # vt2 = 0.02
   # r = 0.1
-  # Bmin = log(1)
+  # Bmin = 0
   # Bmax = log(5)
   # CI.level = 0.95
   # tail = "above"
@@ -794,7 +876,7 @@ sens_plot = function(method="calibrated",
       
       
       if ( is.na(tail) ) {
-        tail = ifelse( yr > log(1), "above", "below" )
+        tail = ifelse( yr > 0, "above", "below" )
         warning( paste( "Assuming you want tail =", tail, "because it wasn't specified") )
       }
       
@@ -854,7 +936,7 @@ sens_plot = function(method="calibrated",
       } else {
         graphics::plot( p + ggplot2::geom_ribbon( aes(ymin=lo, ymax=hi), alpha=0.15 ) )
         
-        warning("Calculating parametric confidence intervals in the plot. For values of Phat that are less than 0.15 or greater than 0.85, these confidence intervals may not perform well.")
+        warning("Calculating parametric confidence intervals in the plot. For values of the proportion that are less than 0.15 or greater than 0.85, these confidence intervals may not perform well.")
       }
       
       
@@ -868,7 +950,7 @@ sens_plot = function(method="calibrated",
         calib = calib_ests( yi = dat[[yi.name]], 
                             sei = sqrt( dat[[vi.name]] ) )
         
-        tail = ifelse( median(calib) > log(1), "above", "below" )
+        tail = ifelse( median(calib) > 0, "above", "below" )
         warning( paste( "Assuming you want tail =", tail, "because it wasn't specified") )
       }
       
@@ -913,7 +995,7 @@ sens_plot = function(method="calibrated",
         ##### Warnings About Missing CIs Due to Boot Failures #####
         # if ALL CI limits are missing
         if ( all( is.na(res$lo) ) ) {
-          message( "None of the pointwise confidence intervals were not estimable via bias-corrected and accelerated bootstrapping, so the confidence band on the plot is omitted. You can try increasing R." )
+          message( "None of the pointwise confidence intervals was estimable via bias-corrected and accelerated bootstrapping, so the confidence band on the plot is omitted. You can try increasing the number of bootstrap iterates or choosing a less extreme threshold." )
           # avoid even trying to plot the CI if it's always NA to avoid geom_ribbon errors later
           give.CI = FALSE
         }
@@ -922,7 +1004,7 @@ sens_plot = function(method="calibrated",
         # outer "if" handles case in which AT LEAST ONE CI limit is NA because of boot failures
         if ( any( !is.na(res$lo) ) & any( !is.na(res$hi) ) ) {
           
-          message( "Some of the pointwise confidence intervals were not estimable via bias-corrected and accelerated bootstrapping, so the confidence band on the plot may not be shown for some values of the bias factor. This usually happens at values with a proportion estimate close to 0 or 1. Otherwise, you can try increasing R." )
+          message( "Some of the pointwise confidence intervals were not estimable via bias-corrected and accelerated bootstrapping, so the confidence band on the plot may not be shown for some values of the bias factor. This usually happens at values with a proportion estimate close to 0 or 1. You can try increasing the number of bootstrap iterates or choosing a less extreme threshold." )
           
           if ( any( res$lo[ !is.na(res$lo) ] > res$Phat[ !is.na(res$lo) ] ) | any( res$hi[ !is.na(res$lo) ] < res$Phat[ !is.na(res$lo) ] ) ) {
             
@@ -934,7 +1016,7 @@ sens_plot = function(method="calibrated",
         
       }
       
-        p = ggplot2::ggplot( data = res,
+      p = ggplot2::ggplot( data = res,
                            aes( x = exp(B),
                                 y = Phat ) ) +
         theme_bw() +
@@ -1013,6 +1095,7 @@ g = Vectorize( function(x) {
   # define transformation in a way that is monotonic over the effective range of B (>1)
   # to avoid ggplot errors in sens_plot
   # helper function for confounded_meta
+  if ( is.na(x) ) return(NA)
   if (x < 1) return( x / 1e10 )
   x + sqrt( x^2 - x )
 } )
@@ -1155,7 +1238,7 @@ Phat_CI_lims = function(.B,
 
 
 
-#' CI for Tmin and Gmini
+#' CI for Tmin and Gmin
 #'
 #' An internal function that estimates a CI for Tmin and Gmin. Users should call \code{confounded_meta} instead.
 #' @import
@@ -1198,6 +1281,10 @@ Tmin_Gmin_CI_lims = function(
     hi.T = bootCIs.Tmin$bca[5]  # but has no upper bound
     SE.T = sd(boot.res$t)
     
+    # avoid issues with creating df below and with g() transformation
+    if ( is.null(lo.T) ) lo.T = NA
+    if ( is.null(hi.T) ) hi.T = NA
+    
     
     ##### Gmin #####
     lo.G = max( 1, g(lo.T) )  # confounding RR can't be < 1
@@ -1206,8 +1293,6 @@ Tmin_Gmin_CI_lims = function(
     
     
     # avoid issues with creating df below
-    if ( is.null(lo.T) ) lo.T = NA
-    if ( is.null(hi.T) ) hi.T = NA
     if ( is.null(lo.G) ) lo.G = NA
     if ( is.null(hi.G) ) hi.G = NA
     
@@ -1248,7 +1333,7 @@ Tmin_Gmin_CI_lims = function(
 
 #' A meta-analysis on soy intake and breast cancer risk (Trock et al., 2006)
 #'
-#' A meta-analysis of observational studies (12 case-control and six cohort or nested case-control) on the association of soy-food intake with breast cancer risk. Data are from Trock et al.'s (2006) Table 1. This dataset was used as the applied example in Mathur & VanderWeele (2019). 
+#' A meta-analysis of observational studies (12 case-control and six cohort or nested case-control) on the association of soy-food intake with breast cancer risk. Data are from Trock et al.'s (2006) Table 1. This dataset was used as the applied example in Mathur & VanderWeele (2020a). 
 #'
 #' @docType data
 #' @keywords datasets
@@ -1266,18 +1351,6 @@ Tmin_Gmin_CI_lims = function(
 "soyMeta"
 
 
-############################ META-ANALYSIS FUNCTIONS ############################ 
-
-
-
-
-
-
-
-
-
-
-
 
 
 
